React = require('react')
PropTypes = require('prop-types')
omit = require('lodash/omit')
validations = require('nebenan-helpers/lib/validations')
{ arrayToHash } = require('nebenan-helpers/lib/utils/data')
InteractiveComponent = require('./interactive_component')

SKIP_PARSING = arrayToHash(['isRequired', 'isRegex'])

parseValidations = (validationsString) ->
  return [] if not validationsString
  throw new Error('validation must be a string') if typeof validationsString isnt 'string'

  for validation in validationsString.split(';')
    [name, args] = validation.split(':')
    continue if SKIP_PARSING[name] or not validations[name]
    args = args?.split(',') or []
    (ownArgs...) -> validations[name](ownArgs..., args.map(JSON.parse)...)

getRegexValidation = (pattern) ->
  regex = new RegExp("^#{pattern}$")
  (value) -> validations.isRegex(value, regex)


class InputComponent extends InteractiveComponent
  @contextTypes:
    locale: PropTypes.object
    form: PropTypes.object

  constructor: (props) ->
    super(props)
    @state = @getDefaultState(props)

  componentDidMount: ->
    super()
    @context.form.addInput(@) if @isConnected()
    { @syncValidations, @asyncValidation } = @getValidations(@props)

  componentWillUnmount: ->
    @context.form.removeInput(@) if @isConnected()
    super()

  componentWillReceiveProps: (nextProps) ->
    { @syncValidations, @asyncValidation } = @getValidations(nextProps)

  validate: =>
    promise = @getValidation(@getValue())
    promise.then(@clearValidationError).catch(@setValidationError)
    promise

  isConnected: -> @context.form and @getName()
  isValid: -> @state.isValid
  isPristine: -> @state.isPristine
  isErrorActive: -> not @isValid() and @getError()
  isRequired: -> @syncValidations.indexOf(validations.isRequired) isnt -1

  getInput: -> @refs.input or @els.input

  getValidations: (props) ->
    syncValidations = []
    asyncValidation = null

    if typeof props.validate is 'function'
      asyncValidation = props.validate
    else
      syncValidations = parseValidations(props.validate)

    if props.required
      syncValidations.push(validations.isRequired)

    if props.pattern
      syncValidations.push(getRegexValidation(props.pattern))

    { syncValidations, asyncValidation }

  getValidation: (value) ->
    hasValue = validations.isRequired(value)

    # Prevent validating empty non-required fields, reject early if input required and empty
    if @isRequired()
      return Promise.reject() if not hasValue
    else
      return Promise.resolve() if not hasValue

    # prevent async validations triggering if sync validations invalid
    for func in @syncValidations
      return Promise.reject() if not func(value)

    return @asyncValidation(value) if @asyncValidation
    Promise.resolve()

  getDefaultState: (props) ->
    isPristine: true
    isValid: true
    error: null
    value: props.defaultValue

  getValidationError: -> @props.error or true
  clearValidationError: => @setError(null)
  setValidationError: => @setError(@getValidationError())

  getError: -> @state.error
  getName: -> @props.name
  getValue: -> @state.value

  reset: (done) ->
    return unless @isComponentMounted
    updater = (state, props) => @getDefaultState(props)
    @setState(updater, done)

  setValue: (value, done, options = {}) ->
    return unless @isComponentMounted

    isPristine = false
    isValid = true
    error = null

    complete = =>
      @props.onUpdate?(@getValue()) if not options.silent

      if Boolean(@getError())
        @props.onError?(@getError()) if not options.silent
        @context.form.updateValidity() if @isConnected()

      done?()

    @setState({ isPristine, isValid, error, value }, complete)

  setError: (error, done) =>
    return unless @isComponentMounted
    isPristine = false
    isValid = not error

    complete = =>
      @props.onError?(@getError())
      @context.form.updateValidity() if @isConnected()
      done?()

    @setState({ isPristine, isValid, error }, complete)

  setPristine: (done) ->
    return unless @isComponentMounted
    updater = (state, props) => omit(@getDefaultState(props), 'value')
    @setState(updater, done)

  actionChange: (action) ->
    (event) =>
      @setValue(event.target.value)
      action?(event)

  actionClearError: (action) ->
    (event) =>
      @setError(null)
      action?(event)

  actionValidate: (action) ->
    (event) =>
      @validate()
      action?(event)

module.exports = InputComponent
